/* Copyright 2020, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "memcard.hpp"
#include "config.h"
#include "app_config.h"
#include "git_version.h"
#include <iostream>
#include <cxxopts.hpp>
#include <string>

namespace {
} //unnamed namespace

int main(int argc, char* argv[]) {
	using std::string;

	cxxopts::Options options(APP_NAME, APP_SHORT_DESC);
	options.add_options()
		("h,help", "Show this help and quit")
		("version", "Print version info")
		("i,input", "Path to the input memory card file", cxxopts::value<string>()->default_value(""))
		("blocknum", "Print the block number for each savegame")
		("usedblocks", "Print how many blocks are used for each savegame if more than 1")
		("pocketstation", "Print a 'P' to mark when a savegame contains PocketStation data")
		("countrycode", "Print the country code of each savegame")
		("productcode", "Print the product code of the game")
		("identifier", "Print the unique savegame identifier")
		("toccheck", "Print the TOC checksum for each savegame")
	;
	auto command = options.parse(argc, argv);

	if (command.count("help")) {
		std::cout << options.help() << std::endl;
		return 0;
	}

	if (command.count("version")) {
		std::cout << PROJECT_NAME << " v" << project_ver()
			<< (sizeof(PROJECT_GIT_SHA1) > 1 ? " git revision " PROJECT_GIT_SHA1 : "")
			<< " built with " COMPILER_NAME " " COMPILER_VERSION
			<< '\n';
		std::cout << "Copyright (C) " COPYRIGHT_YEAR "  Michele Santullo\n";
		std::cout << "This is free software; see the source for copying conditions.  There is NO\n";
		std::cout << "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n";
		std::cout << "To obtain source code please visit:\n -> " << PROJECT_REPO_URL << '\n';

		return 0;
	}

	std::string mc_path = command["input"].as<string>();
	if (mc_path.empty()) {
		std::cerr << "No input memory card specified, run with --help for usage\n";
		return 2;
	}

	{
		PrintOptions opts;
		opts.title = true;
		opts.block_num = command.count("blocknum") > 0;
		opts.used_blocks = command.count("usedblocks") > 0;
		opts.has_pocket_station_content = command.count("pocketstation") > 0;
		opts.country_code = command.count("countrycode") > 0;
		opts.product_code = command.count("productcode") > 0;
		opts.savegame_identifier = command.count("identifier") > 0;
		opts.toc_checksum = command.count("toccheck") > 0;
		print_blocks(mc_path, opts);
	}
	return 0;
}
