/* Copyright 2020, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "memcard.hpp"
#include "memcard/memorycard.hpp"
#include "memcard/make_memory_card.hpp"
#include <iostream>
#include <iomanip>

void print_blocks (const std::string& mc_path, const PrintOptions& opts) {
	using mc::psx::MemoryCard;
	using mc::psx::make_memory_card;

	const MemoryCard mc (make_memory_card(mc_path));
	{
		mc::psx::ContentInfo nfo = mc.content_info();
		if (nfo.magic != nfo.MCMagic) {
			std::cout << "Invalid memory card data\n";
			return;
		}
	}

	for (const auto& blk : mc) {
		if (blk.has_magic()) {
			if (opts.block_num)
				std::cout << "Block " << std::setfill(' ') << std::setw(2) << blk.index() << ": ";
			if (opts.title)
				std::cout << '"' << blk.title() << "\" ";
			if (opts.used_blocks and blk.block_count() > 1)
				std::cout << "(x" << blk.block_count() << ") ";
			if (opts.has_pocket_station_content and blk.has_pocket_station_content())
				std::cout << "P ";
			if (opts.country_code)
				std::cout << '[' << to_char(blk.country_code()) << "] ";
			if (opts.product_code)
				std::cout << blk.product_code() << ' ';
			if (opts.savegame_identifier)
				std::cout << '\'' << blk.identifier() << "' ";
			if (opts.toc_checksum)
				std::cout << "0x" << std::setfill('0') << std::setw(2) <<  std::hex << static_cast<int>(blk.toc_checksum()) << std::dec << ' ';
			std::cout << '\n';
		}
	}
}
