#include "main_window.hpp"
#include "version_window.hpp"
#include "memcard/memorycard.hpp"
#include "make_nana_animation.hpp"
#include "memcard/make_memory_card.hpp"
#include <iostream>
#include <fstream>

namespace mc {
MainWindow::MainWindow (int icon_size, int icon_fps, const std::string& caption) :
	m_grid1(*this, 3, true),
	m_grid2(*this, 3, true),
	m_label1(*this, "Slot 1"),
	m_label2(*this, "Slot 2"),
	m_menubar(*this),
	m_icon_size(icon_size),
	m_icon_fps(icon_fps)
{
	using mc::psx::MemoryCard;

#if !defined(NDEBUG)
	m_grid1.bgcolor(nana::colors::azure);
#endif
	this->caption(caption);
	this->init_menu();

	const MemoryCard mc1(mc::psx::make_memory_card("/home/michele/dev/code/cpp/memoserv/epsxe000_xa2.mcr"));
	const MemoryCard mc2(mc::psx::make_memory_card("/home/michele/dev/code/cpp/memoserv/michele_epsxe000.mcr"));


	//auto mc1_info = mc1.content_info();
	//std::cout << "memory card 1:\n";
	//std::cout << "\tuse byte: 0x" << std::hex << mc1_info.use_byte << std::dec << '\n';
	//std::cout << "\tavailable blocks: 0x" << std::hex << static_cast<int>(mc1_info.available_blocks) << std::dec << '\n';

	for (int z = 0; z < 15; ++z) {
		if (mc1[z].has_magic())
			m_grid1.emplace_back(duck::make_nana_animation(mc1[z], m_icon_size, m_icon_size, m_icon_fps));
		if (mc2[z].has_magic())
			m_grid2.emplace_back(duck::make_nana_animation(mc2[z], m_icon_size, m_icon_size, m_icon_fps));
	}

	this->div("fit margin=3 <vert fit slot1> <width=25> <vert fit slot2>");
	(*this)["slot1"] << m_label1 << m_grid1;
	(*this)["slot2"] << m_label2 << m_grid2;
	this->collocate();

	//const MemoryCard mc(std::ifstream("/home/michele/emu/psx/WipEout 3 - Special Edition (Europe) (En,Fr,De,Es,It).srm", std::ios::binary));
	//MemoryCard mc(std::ifstream("/home/michele/emu/psx/Rapid Racer (Europe) (En,Fr,De,Es,It).srm", std::ios::binary));
}

void MainWindow::init_menu() {
	m_menubar.push_back("&File");
	m_menubar.at(0).append("E&xit", [](nana::menu::item_proxy&){nana::API::exit();});

	m_menubar.push_back("&Help");
	m_menubar.at(1).append("Version info", [this](nana::menu::item_proxy&) {
		VersionWindow win(*this);
		win.show();
		win.modality();
	});
}
} //namespace mc
