/* Copyright 2020, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "nana/gui/widgets/widget.hpp"
#include "animation_with_size.hpp"
#include <list>
#include <memory>

namespace duck::widget {
namespace drawerbase::block_grid {

class Trigger : public nana::drawer_trigger {
	class Measurer;
public:
	Trigger();
	~Trigger() noexcept;

	void set_block_count (std::size_t sz) noexcept;
	void set_icon_size (nana::size sz) noexcept;
	void set_columns (int c) noexcept;
	int columns() const noexcept;

private:
	virtual void attached(nana::widget&, nana::paint::graphics&) override;
	virtual void detached() override;
	virtual void refresh (nana::paint::graphics&) override;
	virtual void mouse_down(nana::paint::graphics&, const nana::arg_mouse&) override;
	virtual void mouse_enter (nana::paint::graphics& graph, const nana::arg_mouse&) override;
	virtual void mouse_leave (nana::paint::graphics& graph, const nana::arg_mouse&) override;
	virtual void mouse_move (nana::paint::graphics& graph, const nana::arg_mouse&) override;

	std::unique_ptr<Measurer> m_measurer;
	nana::widget* m_widget{nullptr};
	nana::paint::graphics* m_graph{nullptr};
	std::size_t m_highlight_index{};
	std::size_t m_block_count{};
	int m_icon_w{64};
	int m_icon_h{64};
	int m_columns{1};
};
} //namespace drawerbase::block_grid

class BlockGrid : public nana::widget_object<nana::category::widget_tag, drawerbase::block_grid::Trigger> {
public:
	BlockGrid() = default;
	BlockGrid (nana::window wd, unsigned int columns, bool visible);
	~BlockGrid() noexcept = default;

	void emplace_back (AnimationWithSize&& anim);

private:
	std::list<AnimationWithSize> m_blocks;
};
} //namespace duck::widget
