/* Copyright 2020, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <cstddef>
#include <cstdint>
#include <type_traits>
#include <algorithm>

namespace mc::psx {
template <bool Const>
class BasicFrame {
	using data_type = typename std::conditional<Const, const uint8_t, uint8_t>::type;
public:
	static const constexpr unsigned int Size = 128;

	BasicFrame (data_type* data, std::size_t index);
	BasicFrame (const BasicFrame&) = default;
	BasicFrame (BasicFrame&&) = default;

	data_type* begin() { return m_data; }
	data_type* end() { return m_data + size(); }
	const data_type* cbegin() const { return m_data; }
	const data_type* cend() const { return m_data + size(); }
	const data_type* begin() const { return m_data; }
	const data_type* end() const { return m_data + size(); }
	data_type& operator[] (std::size_t index);
	const data_type& operator[] (std::size_t index) const;

	static constexpr std::size_t size() { return Size; }

	data_type* data() { return m_data; }
	const data_type* data() const { return m_data; }

	template <typename T>
	T extract (std::size_t offset) const;

private:
	void extract (std::size_t offset, std::size_t sz, uint8_t* out);

	std::size_t m_index;
	data_type* m_data;
};

template <typename T, bool Const>
inline T read_as (const BasicFrame<Const>& frame, std::size_t offset) {
	T retval;
	std::copy(
		frame.begin() + offset,
		frame.begin() + offset + sizeof(T),
		reinterpret_cast<uint8_t*>(&retval)
	);
	return retval;
}

using Frame = BasicFrame<false>;
using ConstFrame = BasicFrame<true>;
} //namespace mc::psx
