/* Copyright 2020, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "memcard/block.hpp"
#include "memcard/content_info.hpp"
#include "memcard/block_iterator.hpp"
#include <vector>
#include <cstdint>
#include <array>
#include <cstddef>
#include <algorithm>

//see: https://www.psdevwiki.com/ps3/PS1_Savedata#Memory_Card_Formats_PS1

namespace mc::psx {
class MemoryCard {
public:
	typedef BlockIterator<false> iterator;
	typedef BlockIterator<true> const_iterator;

	MemoryCard();
	template <typename IT> explicit MemoryCard (IT beg, IT end);
	~MemoryCard();

	Block operator[] (std::size_t index);
	ConstBlock operator[] (std::size_t index) const;
	ContentInfo content_info() const;

	std::size_t size() const;

	iterator begin();
	iterator end();
	const_iterator cbegin() const;
	const_iterator begin() const;
	const_iterator cend() const;
	const_iterator end() const;

private:
	Block block_at_index(std::size_t index);
	ConstBlock block_at_index(std::size_t index) const;

	std::vector<uint8_t> m_data;
};

std::vector<std::size_t> block_group (const MemoryCard& mc, std::size_t index);

template <typename IT>
MemoryCard::MemoryCard (IT beg, IT end) :
	MemoryCard()
{
	std::copy_if(
		beg,
		end,
		m_data.begin(),
		[count = m_data.size()](auto&&) mutable {return static_cast<bool>(count--);}
	);
}

} //namespace mc::psx
