/* Copyright 2020, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "memcard/content_info.hpp"
#include "memcard/frame.hpp"
#include <algorithm>

namespace mc::psx {
namespace {
} //unnamed namespace

ContentInfo::ContentInfo (const ConstFrame& frame) :
	use_byte(read_as<uint32_t>(frame, 0x84)),
	magic(read_as<uint16_t>(frame, 0x00)),
	available_blocks(BlockContent::Available)
{
	auto block_content = read_as<uint8_t>(frame, 0x80);
	switch (block_content & 0xf) {
	case 0x0:
		available_blocks = BlockContent::Available;
		break;
	case 0x1:
		available_blocks = BlockContent::UsedLinkNextBlock;
		break;
	case 0x2:
		available_blocks = BlockContent::UsedMidLink;
		break;
	case 0x3:
		available_blocks = BlockContent::UsedEndLink;
		break;
	case 0xf:
	default:
		available_blocks = BlockContent::Unusable;
		break;
	}
}
} //namespace mc::psx
